# Copyright 2016 Red Hat, Inc.
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient.tests.functional import base


class TestNetworkCommandsV2_36(base.ClientTestBase):
    """Deprecated network command functional tests."""

    # Proxy APIs were deprecated in 2.36 but the CLI should fallback to 2.35
    # and emit a warning.
    COMPUTE_API_VERSION = "2.36"

    def test_limits(self):
        """Tests that 2.36 won't return network-related resource limits and
        the CLI output won't show them.
        """
        output = self.nova('limits')
        # assert that SecurityGroups isn't in the table output
        self.assertRaises(ValueError, self._get_value_from_the_table,
                          output, 'SecurityGroups')

    def test_quota_show(self):
        """Tests that 2.36 won't return network-related resource quotas and
        the CLI output won't show them.
        """
        output = self.nova('quota-show')
        # assert that security_groups isn't in the table output
        self.assertRaises(ValueError, self._get_value_from_the_table,
                          output, 'security_groups')



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Hypervisors interface
"""

from urllib import parse

from novaclient import api_versions
from novaclient import base
from novaclient import exceptions
from novaclient.i18n import _
from novaclient import utils


class Hypervisor(base.Resource):
    NAME_ATTR = 'hypervisor_hostname'

    def __repr__(self):
        return "<Hypervisor: %s>" % self.id


class HypervisorManager(base.ManagerWithFind):
    resource_class = Hypervisor
    is_alphanum_id_allowed = True

    def _list_base(self, detailed=True, marker=None, limit=None):
        path = '/os-hypervisors'
        if detailed:
            path += '/detail'
        params = {}
        if limit is not None:
            params['limit'] = int(limit)
        if marker is not None:
            params['marker'] = str(marker)
        path += utils.prepare_query_string(params)
        return self._list(path, 'hypervisors')

    @api_versions.wraps("2.0", "2.32")
    def list(self, detailed=True):
        """
        Get a list of hypervisors.

        :param detailed: Include a detailed response.
        """
        return self._list_base(detailed=detailed)

    @api_versions.wraps("2.33")
    def list(self, detailed=True, marker=None, limit=None):
        """
        Get a list of hypervisors.

        :param detailed: Include a detailed response.
        :param marker: Begin returning hypervisors that appear later in the
                       hypervisors list than that represented by this
                       hypervisor ID. Starting with microversion 2.53 the
                       marker must be a UUID hypervisor ID.
                       (optional).
        :param limit: maximum number of hypervisors to return (optional).
                      Note the API server has a configurable default limit.
                      If no limit is specified here or limit is larger than
                      default, the default limit will be used.
        """
        return self._list_base(detailed=detailed, marker=marker, limit=limit)

    def search(self, hypervisor_match, servers=False, detailed=False):
        """
        Get a list of matching hypervisors.

        :param hypervisor_match: The hypervisor host name or a portion of it.
            The hypervisor hosts are selected with the host name matching
            this pattern.
        :param servers: If True, server information is also retrieved.
        :param detailed: If True, detailed hypervisor information is returned.
            This requires API version 2.53 or greater.
        """
        # Starting with microversion 2.53, the /servers and /search routes are
        # deprecated and we get the same results using GET /os-hypervisors
        # using query parameters for the hostname pattern and servers.
        if self.api_version >= api_versions.APIVersion('2.53'):
            url = ('/os-hypervisors%s?hypervisor_hostname_pattern=%s' %
                   ('/detail' if detailed else '',
                    parse.quote(hypervisor_match, safe='')))
            if servers:
                url += '&with_servers=True'
        else:
            if detailed:
                raise exceptions.UnsupportedVersion(
                    _('Parameter "detailed" requires API version 2.53 or '
                      'greater.'))
            target = 'servers' if servers else 'search'
            url = ('/os-hypervisors/%s/%s' %
                   (parse.quote(hypervisor_match, safe=''), target))
        return self._list(url, 'hypervisors')

    def get(self, hypervisor):
        """
        Get a specific hypervisor.

        :param hypervisor: Either a Hypervisor object or an ID. Starting with
            microversion 2.53 the ID must be a UUID value.
        """
        return self._get("/os-hypervisors/%s" % base.getid(hypervisor),
                         "hypervisor")

    def uptime(self, hypervisor):
        """
        Get the uptime for a specific hypervisor.

        :param hypervisor: Either a Hypervisor object or an ID. Starting with
            microversion 2.53 the ID must be a UUID value.
        """
        # Starting with microversion 2.88, the '/os-hypervisors/{id}/uptime'
        # route is removed in favour of returning 'uptime' in the response of
        # the '/os-hypervisors/{id}' route. This behaves slightly differently,
        # in that it won't error out if a virt driver doesn't support reporting
        # uptime or if the hypervisor is down, but it's a good enough
        # approximation
        if self.api_version < api_versions.APIVersion("2.88"):
            return self._get(
                "/os-hypervisors/%s/uptime" % base.getid(hypervisor),
                "hypervisor")

        resp, body = self.api.client.get(
            "/os-hypervisors/%s" % base.getid(hypervisor)
        )
        content = {
            k: v for k, v in body['hypervisor'].items()
            if k in ('id', 'hypervisor_hostname', 'state', 'status', 'uptime')
        }
        return self.resource_class(self, content, loaded=True, resp=resp)

    def statistics(self):
        """
        Get hypervisor statistics over all compute nodes.

        Kept for backwards compatibility, new code should call
        hypervisor_stats.statistics() instead of hypervisors.statistics()
        """
        return self.api.hypervisor_stats.statistics()


class HypervisorStats(base.Resource):
    def __repr__(self):
        return ("<HypervisorStats: %d Hypervisor%s>" %
                (self.count, "s" if self.count != 1 else ""))


class HypervisorStatsManager(base.Manager):
    resource_class = HypervisorStats

    @api_versions.wraps("2.0", "2.87")
    def statistics(self):
        """
        Get hypervisor statistics over all compute nodes.
        """
        return self._get("/os-hypervisors/statistics", "hypervisor_statistics")

    @api_versions.wraps("2.88")
    def statistics(self):
        raise exceptions.UnsupportedVersion(
            _("The 'statistics' API is removed in API version 2.88 or later.")
        )



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import importlib
import inspect
from unittest import mock

import stevedore
from stevedore import extension

from novaclient import client
from novaclient.tests.unit import utils


class DiscoverTest(utils.TestCase):

    def test_discover_via_entry_points(self):

        def mock_mgr():
            fake_ep = mock.Mock()
            fake_ep.name = 'foo'
            module_spec = importlib.machinery.ModuleSpec('foo', None)
            fake_ep.module = importlib.util.module_from_spec(module_spec)
            fake_ep.load.return_value = fake_ep.module
            fake_ext = extension.Extension(
                name='foo',
                entry_point=fake_ep,
                plugin=fake_ep.module,
                obj=None,
            )
            return stevedore.ExtensionManager.make_test_instance([fake_ext])

        @mock.patch.object(client, '_make_discovery_manager', mock_mgr)
        def test():
            for name, module in client._discover_via_entry_points():
                self.assertEqual('foo', name)
                self.assertTrue(inspect.ismodule(module))

        test()

    def test_discover_extensions(self):

        def mock_discover_via_python_path():
            module_spec = importlib.machinery.ModuleSpec('foo', None)
            module = importlib.util.module_from_spec(module_spec)
            yield 'foo', module

        def mock_discover_via_entry_points():
            module_spec = importlib.machinery.ModuleSpec('baz', None)
            module = importlib.util.module_from_spec(module_spec)
            yield 'baz', module

        @mock.patch.object(client,
                           '_discover_via_python_path',
                           mock_discover_via_python_path)
        @mock.patch.object(client,
                           '_discover_via_entry_points',
                           mock_discover_via_entry_points)
        def test():
            extensions = client.discover_extensions('1.1')
            self.assertEqual(2, len(extensions))
            names = sorted(['foo', 'baz'])
            sorted_extensions = sorted(extensions, key=lambda ext: ext.name)
            for i in range(len(names)):
                ext = sorted_extensions[i]
                name = names[i]
                self.assertEqual(ext.name, name)
                self.assertTrue(inspect.ismodule(ext.module))

        test()



# Copyright 2012 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient import api_versions
from novaclient.tests.unit import utils
from novaclient.tests.unit.v2 import fakes
from novaclient.v2 import services


class ServicesTest(utils.TestCase):
    api_version = "2.0"

    def setUp(self):
        super(ServicesTest, self).setUp()
        self.cs = fakes.FakeClient(api_versions.APIVersion(self.api_version))
        self.service_type = self._get_service_type()

    def _get_service_type(self):
        return services.Service

    def test_list_services(self):
        svs = self.cs.services.list()
        self.assert_request_id(svs, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET', '/os-services')
        for s in svs:
            self.assertIsInstance(s, self._get_service_type())
            self.assertEqual('nova-compute', s.binary)
            self.assertEqual('host1', s.host)
            self.assertEqual('<Service: %s>' % s.id, str(s))

    def test_list_services_with_hostname(self):
        svs = self.cs.services.list(host='host2')
        self.assert_request_id(svs, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET', '/os-services?host=host2')
        for s in svs:
            self.assertIsInstance(s, self._get_service_type())
            self.assertEqual('nova-compute', s.binary)
            self.assertEqual('host2', s.host)

    def test_list_services_with_binary(self):
        svs = self.cs.services.list(binary='nova-cert')
        self.assert_request_id(svs, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET', '/os-services?binary=nova-cert')
        for s in svs:
            self.assertIsInstance(s, self._get_service_type())
            self.assertEqual('nova-cert', s.binary)
            self.assertEqual('host1', s.host)

    def test_list_services_with_host_binary(self):
        svs = self.cs.services.list(host='host2', binary='nova-cert')
        self.assert_request_id(svs, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET',
                              '/os-services?host=host2&binary=nova-cert')
        for s in svs:
            self.assertIsInstance(s, self._get_service_type())
            self.assertEqual('nova-cert', s.binary)
            self.assertEqual('host2', s.host)

    def _update_body(self, host, binary, disabled_reason=None):
        body = {"host": host,
                "binary": binary}
        if disabled_reason is not None:
            body["disabled_reason"] = disabled_reason
        return body

    def test_services_enable(self):
        service = self.cs.services.enable('host1', 'nova-cert')
        self.assert_request_id(service, fakes.FAKE_REQUEST_ID_LIST)
        values = self._update_body("host1", "nova-cert")
        self.cs.assert_called('PUT', '/os-services/enable', values)
        self.assertIsInstance(service, self._get_service_type())
        self.assertEqual('enabled', service.status)

    def test_services_delete(self):
        ret = self.cs.services.delete('1')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('DELETE', '/os-services/1')

    def test_services_disable(self):
        service = self.cs.services.disable('host1', 'nova-cert')
        self.assert_request_id(service, fakes.FAKE_REQUEST_ID_LIST)
        values = self._update_body("host1", "nova-cert")
        self.cs.assert_called('PUT', '/os-services/disable', values)
        self.assertIsInstance(service, self._get_service_type())
        self.assertEqual('disabled', service.status)

    def test_services_disable_log_reason(self):
        service = self.cs.services.disable_log_reason(
            'compute1', 'nova-compute', 'disable bad host')
        self.assert_request_id(service, fakes.FAKE_REQUEST_ID_LIST)
        values = self._update_body("compute1", "nova-compute",
                                   "disable bad host")
        self.cs.assert_called('PUT', '/os-services/disable-log-reason', values)
        self.assertIsInstance(service, self._get_service_type())
        self.assertEqual('disabled', service.status)


class ServicesV211TestCase(ServicesTest):
    api_version = "2.11"

    def _update_body(self, host, binary, disabled_reason=None,
                     force_down=None):
        body = {"host": host,
                "binary": binary}
        if disabled_reason is not None:
            body["disabled_reason"] = disabled_reason
        if force_down is not None:
            body["forced_down"] = force_down
        return body

    def test_services_force_down(self):
        service = self.cs.services.force_down(
            'compute1', 'nova-compute', False)
        self.assert_request_id(service, fakes.FAKE_REQUEST_ID_LIST)
        values = self._update_body("compute1", "nova-compute",
                                   force_down=False)
        self.cs.assert_called('PUT', '/os-services/force-down', values)
        self.assertIsInstance(service, self._get_service_type())
        self.assertFalse(service.forced_down)


class ServicesV2_53TestCase(ServicesV211TestCase):
    api_version = "2.53"

    def _update_body(self, status=None, disabled_reason=None, force_down=None):
        body = {}
        if status is not None:
            body['status'] = status
        if disabled_reason is not None:
            body['disabled_reason'] = disabled_reason
        if force_down is not None:
            body['forced_down'] = force_down
        return body

    def test_services_enable(self):
        service = self.cs.services.enable(fakes.FAKE_SERVICE_UUID_1)
        self.assert_request_id(service, fakes.FAKE_REQUEST_ID_LIST)
        values = self._update_body(status='enabled')
        self.cs.assert_called(
            'PUT', '/os-services/%s' % fakes.FAKE_SERVICE_UUID_1, values)
        self.assertIsInstance(service, self._get_service_type())
        self.assertEqual('enabled', service.status)

    def test_services_delete(self):
        ret = self.cs.services.delete(fakes.FAKE_SERVICE_UUID_1)
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('DELETE',
                              '/os-services/%s' % fakes.FAKE_SERVICE_UUID_1)

    def test_services_disable(self):
        service = self.cs.services.disable(fakes.FAKE_SERVICE_UUID_1)
        self.assert_request_id(service, fakes.FAKE_REQUEST_ID_LIST)
        values = self._update_body(status='disabled')
        self.cs.assert_called(
            'PUT', '/os-services/%s' % fakes.FAKE_SERVICE_UUID_1, values)
        self.assertIsInstance(service, self._get_service_type())
        self.assertEqual('disabled', service.status)

    def test_services_disable_log_reason(self):
        service = self.cs.services.disable_log_reason(
            fakes.FAKE_SERVICE_UUID_1, 'disable bad host')
        self.assert_request_id(service, fakes.FAKE_REQUEST_ID_LIST)
        values = self._update_body(status='disabled',
                                   disabled_reason='disable bad host')
        self.cs.assert_called(
            'PUT', '/os-services/%s' % fakes.FAKE_SERVICE_UUID_1, values)
        self.assertIsInstance(service, self._get_service_type())
        self.assertEqual('disabled', service.status)
        self.assertEqual('disable bad host', service.disabled_reason)

    def test_services_force_down(self):
        service = self.cs.services.force_down(
            fakes.FAKE_SERVICE_UUID_1, False)
        self.assert_request_id(service, fakes.FAKE_REQUEST_ID_LIST)
        values = self._update_body(force_down=False)
        self.cs.assert_called(
            'PUT', '/os-services/%s' % fakes.FAKE_SERVICE_UUID_1, values)
        self.assertIsInstance(service, self._get_service_type())
        self.assertFalse(service.forced_down)



# Copyright 2010 Jacob Kaplan-Moss
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Exception definitions.
"""


class UnsupportedVersion(Exception):
    """Indicates that the user is trying to use an unsupported
    version of the API.
    """
    pass


class UnsupportedConsoleType(Exception):
    """Indicates that the user is trying to use an unsupported
    console type when retrieving console urls of servers.
    """
    def __init__(self, console_type):
        self.message = 'Unsupported console_type "%s"' % console_type


class UnsupportedAttribute(AttributeError):
    """Indicates that the user is trying to transmit the argument to a method,
    which is not supported by selected version.
    """

    def __init__(self, argument_name, start_version, end_version=None):
        if end_version:
            self.message = (
                "'%(name)s' argument is only allowed for microversions "
                "%(start)s - %(end)s." % {"name": argument_name,
                                          "start": start_version,
                                          "end": end_version})
        else:
            self.message = (
                "'%(name)s' argument is only allowed since microversion "
                "%(start)s." % {"name": argument_name, "start": start_version})
        super(UnsupportedAttribute, self).__init__(self.message)


class CommandError(Exception):
    pass


class NoUniqueMatch(Exception):
    pass


class ResourceInErrorState(Exception):
    """Resource is in the error state."""

    def __init__(self, obj):
        msg = "`%s` resource is in the error state" % obj.__class__.__name__
        fault_msg = getattr(obj, "fault", {}).get("message")
        if fault_msg:
            msg += "due to '%s'" % fault_msg
        self.message = "%s." % msg


class VersionNotFoundForAPIMethod(Exception):
    msg_fmt = "API version '%(vers)s' is not supported on '%(method)s' method."

    def __init__(self, version, method):
        self.version = version
        self.method = method

    def __str__(self):
        return self.msg_fmt % {"vers": self.version, "method": self.method}


class InstanceInDeletedState(Exception):
    """Instance is in the deleted state."""
    pass


class ClientException(Exception):
    """
    The base exception class for all exceptions this library raises.
    """
    message = 'Unknown Error'

    def __init__(self, code, message=None, details=None, request_id=None,
                 url=None, method=None):
        self.code = code
        self.message = message or self.__class__.message
        self.details = details
        self.request_id = request_id
        self.url = url
        self.method = method

    def __str__(self):
        formatted_string = "%s (HTTP %s)" % (self.message, self.code)
        if self.request_id:
            formatted_string += " (Request-ID: %s)" % self.request_id

        return formatted_string


class RetryAfterException(ClientException):
    """
    The base exception class for ClientExceptions that use Retry-After header.
    """
    def __init__(self, *args, **kwargs):
        try:
            self.retry_after = int(kwargs.pop('retry_after'))
        except (KeyError, ValueError):
            self.retry_after = 0

        super(RetryAfterException, self).__init__(*args, **kwargs)


class BadRequest(ClientException):
    """
    HTTP 400 - Bad request: you sent some malformed data.
    """
    http_status = 400
    message = "Bad request"


class Unauthorized(ClientException):
    """
    HTTP 401 - Unauthorized: bad credentials.
    """
    http_status = 401
    message = "Unauthorized"


class Forbidden(ClientException):
    """
    HTTP 403 - Forbidden: your credentials don't give you access to this
    resource.
    """
    http_status = 403
    message = "Forbidden"


class NotFound(ClientException):
    """
    HTTP 404 - Not found
    """
    http_status = 404
    message = "Not found"


class MethodNotAllowed(ClientException):
    """
    HTTP 405 - Method Not Allowed
    """
    http_status = 405
    message = "Method Not Allowed"


class NotAcceptable(ClientException):
    """
    HTTP 406 - Not Acceptable
    """
    http_status = 406
    message = "Not Acceptable"


class Conflict(ClientException):
    """
    HTTP 409 - Conflict
    """
    http_status = 409
    message = "Conflict"


class OverLimit(RetryAfterException):
    """
    HTTP 413 - Over limit: you're over the API limits for this time period.
    """
    http_status = 413
    message = "Over limit"


class RateLimit(RetryAfterException):
    """
    HTTP 429 - Rate limit: you've sent too many requests for this time period.
    """
    http_status = 429
    message = "Rate limit"


# NotImplemented is a python keyword.
class HTTPNotImplemented(ClientException):
    """
    HTTP 501 - Not Implemented: the server does not support this operation.
    """
    http_status = 501
    message = "Not Implemented"


# In Python 2.4 Exception is old-style and thus doesn't have a __subclasses__()
# so we can do this:
#     _code_map = dict((c.http_status, c)
#                      for c in ClientException.__subclasses__())
#
# Instead, we have to hardcode it:
_error_classes = [BadRequest, Unauthorized, Forbidden, NotFound,
                  MethodNotAllowed, NotAcceptable, Conflict, OverLimit,
                  RateLimit, HTTPNotImplemented]
_code_map = dict((c.http_status, c) for c in _error_classes)


class InvalidUsage(RuntimeError):
    """This function call is invalid in the way you are using this client.

    Due to the transition to using keystoneauth some function calls are no
    longer available. You should make a similar call to the session object
    instead.
    """
    pass


def from_response(response, body, url, method=None):
    """
    Return an instance of an ClientException or subclass
    based on a requests response.

    Usage::

        resp, body = requests.request(...)
        if resp.status_code != 200:
            raise exception_from_response(resp, rest.text)
    """
    cls = _code_map.get(response.status_code, ClientException)

    kwargs = {
        'code': response.status_code,
        'method': method,
        'url': url,
        'request_id': None,
    }

    if response.headers:
        kwargs['request_id'] = response.headers.get('x-compute-request-id')

        if (issubclass(cls, RetryAfterException) and
                'retry-after' in response.headers):
            kwargs['retry_after'] = response.headers.get('retry-after')

    if body:
        message = "n/a"
        details = "n/a"

        if hasattr(body, 'keys'):
            # NOTE(mriedem): WebOb<1.6.0 will return a nested dict structure
            # where the error keys to the message/details/code. WebOb>=1.6.0
            # returns just a response body as a single dict, not nested,
            # so we have to handle both cases (since we can't trust what we're
            # given with content_type: application/json either way.
            if 'message' in body:
                # WebOb 1.6.0 case
                message = body.get('message')
                details = body.get('details')
            else:
                # WebOb<1.6.0 where we assume there is a single error message
                # key to the body that has the message and details.
                error = body[list(body)[0]]
                message = error.get('message')
                details = error.get('details')

        kwargs['message'] = message
        kwargs['details'] = details

    return cls(**kwargs)


class ResourceNotFound(Exception):
    """Error in getting the resource."""
    pass



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient.tests.functional import base


class TestFlavorNovaClientV274(base.ClientTestBase):
    """Functional tests for flavors"""

    COMPUTE_API_VERSION = "2.74"
    # NOTE(gmann): Before microversion 2.75, default value of 'swap' field is
    # returned as empty string.
    SWAP_DEFAULT = ""

    def _create_flavor(self, swap=None):
        flv_name = self.name_generate()
        cmd = 'flavor-create %s auto 512 1 1'
        if swap:
            cmd = cmd + (' --swap %s' % swap)
        out = self.nova(cmd % flv_name)
        self.addCleanup(self.nova, 'flavor-delete %s' % flv_name)
        return out, flv_name

    def test_create_flavor_with_no_swap(self):
        out, _ = self._create_flavor()
        self.assertEqual(
            self.SWAP_DEFAULT,
            self._get_column_value_from_single_row_table(out, "Swap"))

    def test_update_flavor_with_no_swap(self):
        _, flv_name = self._create_flavor()
        out = self.nova('flavor-update %s new-description' % flv_name)
        self.assertEqual(
            self.SWAP_DEFAULT,
            self._get_column_value_from_single_row_table(out, "Swap"))

    def test_show_flavor_with_no_swap(self):
        _, flv_name = self._create_flavor()
        out = self.nova('flavor-show %s' % flv_name)
        self.assertEqual(self.SWAP_DEFAULT,
                         self._get_value_from_the_table(out, "swap"))

    def test_list_flavor_with_no_swap(self):
        self._create_flavor()
        out = self.nova('flavor-list')
        self.assertEqual(
            self.SWAP_DEFAULT,
            self._get_column_value_from_single_row_table(out, "Swap"))

    def test_create_flavor_with_swap(self):
        out, _ = self._create_flavor(swap=10)
        self.assertEqual(
            '10',
            self._get_column_value_from_single_row_table(out, "Swap"))


class TestFlavorNovaClientV275(TestFlavorNovaClientV274):
    """Functional tests for flavors"""

    COMPUTE_API_VERSION = "2.75"
    # NOTE(gmann): Since microversion 2.75, default value of 'swap' field is
    # returned as 0.
    SWAP_DEFAULT = '0'



# Copyright 2013 Rackspace Hosting
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient import api_versions
from novaclient.tests.unit import utils
from novaclient.tests.unit.v2 import fakes
from novaclient.v2 import instance_action


class InstanceActionExtensionTests(utils.TestCase):
    def setUp(self):
        super(InstanceActionExtensionTests, self).setUp()
        self.cs = fakes.FakeClient(api_versions.APIVersion("2.1"))

    def test_list_instance_actions(self):
        server_uuid = '1234'
        ial = self.cs.instance_action.list(server_uuid)
        self.assert_request_id(ial, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called(
            'GET', '/servers/%s/os-instance-actions' %
            server_uuid)

    def test_get_instance_action(self):
        server_uuid = '1234'
        request_id = 'req-abcde12345'
        ia = self.cs.instance_action.get(server_uuid, request_id)
        self.assert_request_id(ia, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called(
            'GET', '/servers/%s/os-instance-actions/%s'
            % (server_uuid, request_id))


class InstanceActionExtensionV258Tests(InstanceActionExtensionTests):
    def setUp(self):
        super(InstanceActionExtensionV258Tests, self).setUp()
        self.cs.api_version = api_versions.APIVersion("2.58")

    def test_list_instance_actions_with_limit_marker_params(self):
        server_uuid = '1234'
        marker = '12140183-c814-4ddf-8453-6df43028aa67'

        ias = self.cs.instance_action.list(
            server_uuid, marker=marker, limit=10,
            changes_since='2016-02-29T06:23:22')
        self.assert_request_id(ias, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called(
            'GET',
            '/servers/%s/os-instance-actions?changes-since=%s&limit=10&'
            'marker=%s' % (server_uuid, '2016-02-29T06%3A23%3A22', marker))
        for ia in ias:
            self.assertIsInstance(ia, instance_action.InstanceAction)


class InstanceActionExtensionV266Tests(InstanceActionExtensionV258Tests):
    def setUp(self):
        super(InstanceActionExtensionV266Tests, self).setUp()
        self.cs.api_version = api_versions.APIVersion("2.66")

    def test_list_instance_actions_with_changes_before(self):
        server_uuid = '1234'

        ias = self.cs.instance_action.list(
            server_uuid, marker=None, limit=None, changes_since=None,
            changes_before='2016-02-29T06:23:22')
        self.assert_request_id(ias, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called(
            'GET',
            '/servers/%s/os-instance-actions?changes-before=%s' %
            (server_uuid, '2016-02-29T06%3A23%3A22'))
        for ia in ias:
            self.assertIsInstance(ia, instance_action.InstanceAction)



# Copyright 2010 Jacob Kaplan-Moss
# Copyright 2011 OpenStack Foundation
# Copyright 2011 Piston Cloud Computing, Inc.

# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
OpenStack Client interface. Handles the REST calls and responses.
"""

import itertools
import pkgutil
import warnings

from keystoneauth1 import adapter
from keystoneauth1 import identity
from keystoneauth1 import session as ksession
from oslo_utils import importutils
import stevedore

import novaclient
from novaclient import api_versions
from novaclient import exceptions
from novaclient import extension as ext
from novaclient.i18n import _
from novaclient import utils

osprofiler_profiler = importutils.try_import("osprofiler.profiler")
osprofiler_web = importutils.try_import("osprofiler.web")


class SessionClient(adapter.LegacyJsonAdapter):

    client_name = 'python-novaclient'
    client_version = novaclient.__version__

    def __init__(self, *args, **kwargs):
        self.times = []
        self.timings = kwargs.pop('timings', False)
        self.api_version = kwargs.pop('api_version', None)
        self.api_version = self.api_version or api_versions.APIVersion()
        super(SessionClient, self).__init__(*args, **kwargs)

    def request(self, url, method, **kwargs):
        kwargs.setdefault('headers', kwargs.get('headers', {}))
        api_versions.update_headers(kwargs["headers"], self.api_version)

        # NOTE(dbelova): osprofiler_web.get_trace_id_headers does not add any
        # headers in case if osprofiler is not initialized.
        if osprofiler_web:
            kwargs['headers'].update(osprofiler_web.get_trace_id_headers())

        # NOTE(jamielennox): The standard call raises errors from
        # keystoneauth1, where we need to raise the novaclient errors.
        raise_exc = kwargs.pop('raise_exc', True)
        with utils.record_time(self.times, self.timings, method, url):
            resp, body = super(SessionClient, self).request(url,
                                                            method,
                                                            raise_exc=False,
                                                            **kwargs)

        # TODO(andreykurilin): uncomment this line, when we will be able to
        #   check only nova-related calls
        # api_versions.check_headers(resp, self.api_version)
        if raise_exc and resp.status_code >= 400:
            raise exceptions.from_response(resp, body, url, method)

        return resp, body

    def get_timings(self):
        return self.times

    def reset_timings(self):
        self.times = []


def _construct_http_client(api_version=None,
                           auth=None,
                           auth_token=None,
                           auth_url=None,
                           cacert=None,
                           cert=None,
                           endpoint_override=None,
                           endpoint_type='publicURL',
                           http_log_debug=False,
                           insecure=False,
                           logger=None,
                           os_cache=False,
                           password=None,
                           project_domain_id=None,
                           project_domain_name=None,
                           project_id=None,
                           project_name=None,
                           region_name=None,
                           service_name=None,
                           service_type='compute',
                           session=None,
                           timeout=None,
                           timings=False,
                           user_agent='python-novaclient',
                           user_domain_id=None,
                           user_domain_name=None,
                           user_id=None,
                           username=None,
                           **kwargs):
    if not session:
        if not auth and auth_token:
            auth = identity.Token(auth_url=auth_url,
                                  token=auth_token,
                                  project_id=project_id,
                                  project_name=project_name,
                                  project_domain_id=project_domain_id,
                                  project_domain_name=project_domain_name)
        elif not auth:
            auth = identity.Password(username=username,
                                     user_id=user_id,
                                     password=password,
                                     project_id=project_id,
                                     project_name=project_name,
                                     auth_url=auth_url,
                                     project_domain_id=project_domain_id,
                                     project_domain_name=project_domain_name,
                                     user_domain_id=user_domain_id,
                                     user_domain_name=user_domain_name)
        session = ksession.Session(auth=auth,
                                   verify=(cacert or not insecure),
                                   timeout=timeout,
                                   cert=cert,
                                   user_agent=user_agent)

    return SessionClient(api_version=api_version,
                         auth=auth,
                         endpoint_override=endpoint_override,
                         interface=endpoint_type,
                         logger=logger,
                         region_name=region_name,
                         service_name=service_name,
                         service_type=service_type,
                         session=session,
                         timings=timings,
                         user_agent=user_agent,
                         **kwargs)


def discover_extensions(*args, **kwargs):
    """Returns the list of extensions, which can be discovered by python path
    and by entry-point 'novaclient.extension'.
    """
    chain = itertools.chain(_discover_via_python_path(),
                            _discover_via_entry_points())
    return [ext.Extension(name, module) for name, module in chain]


def _discover_via_python_path():
    for (module_loader, name, _ispkg) in pkgutil.iter_modules():
        if name.endswith('_python_novaclient_ext'):
            # NOTE(sdague): needed for python 2.x compatibility.
            if not hasattr(module_loader, 'load_module'):
                module_loader = module_loader.find_module(name)
            module = module_loader.load_module(name)
            if hasattr(module, 'extension_name'):
                name = module.extension_name

            yield name, module


def _make_discovery_manager():
    # This function provides a place to mock out the entry point scan
    return stevedore.ExtensionManager('novaclient.extension')


def _discover_via_entry_points():
    mgr = _make_discovery_manager()
    for extension in mgr:
        yield extension.name, extension.plugin


def _get_client_class_and_version(version):
    if not isinstance(version, api_versions.APIVersion):
        version = api_versions.get_api_version(version)
    else:
        api_versions.check_major_version(version)
    if version.is_latest():
        raise exceptions.UnsupportedVersion(
            _("The version should be explicit, not latest."))
    return version, importutils.import_class(
        "novaclient.v%s.client.Client" % version.ver_major)


def _check_arguments(kwargs, release, deprecated_name, right_name=None):
    """Process deprecation of arguments.

    Checks presence of deprecated argument in kwargs, prints proper warning
    message, renames key to right one it needed.
    """
    if deprecated_name in kwargs:
        if right_name:
            if right_name in kwargs:
                msg = _("The '%(old)s' argument is deprecated in "
                        "%(release)s and its use may result in errors "
                        "in future releases. As '%(new)s' is provided, "
                        "the '%(old)s' argument will be ignored.") % {
                    "old": deprecated_name, "release": release,
                    "new": right_name}
                kwargs.pop(deprecated_name)
            else:
                msg = _("The '%(old)s' argument is deprecated in "
                        "%(release)s and its use may result in errors in "
                        "future releases. Use '%(right)s' instead.") % {
                    "old": deprecated_name, "release": release,
                    "right": right_name}
                kwargs[right_name] = kwargs.pop(deprecated_name)

        else:
            msg = _("The '%(old)s' argument is deprecated in %(release)s "
                    "and its use may result in errors in future "
                    "releases.") % {
                "old": deprecated_name, "release": release}
            # just ignore it
            kwargs.pop(deprecated_name)

        warnings.warn(msg)


def Client(version, username=None, password=None, project_id=None,
           auth_url=None, **kwargs):
    """Initialize client object based on given version.

    HOW-TO:
    The simplest way to create a client instance is initialization with your
    credentials::

        >>> from novaclient import client
        >>> nova = client.Client(VERSION, USERNAME, PASSWORD,
        ...                      PROJECT_ID, AUTH_URL)

    Here ``VERSION`` can be a string or
    ``novaclient.api_versions.APIVersion`` obj. If you prefer string value,
    you can use ``1.1`` (deprecated now), ``2`` or ``2.X``
    (where X is a microversion).


    Alternatively, you can create a client instance using the keystoneauth
    session API. See "The novaclient Python API" page at
    python-novaclient's doc.
    """
    if password:
        kwargs["password"] = password
    if project_id:
        kwargs["project_id"] = project_id

    _check_arguments(kwargs, "Ocata", "auth_plugin")
    _check_arguments(kwargs, "Ocata", "auth_system")
    if "no_cache" in kwargs:
        _check_arguments(kwargs, "Ocata", "no_cache", right_name="os_cache")
        # os_cache is not a fully compatible with no_cache, so we need to
        # apply this custom processing
        kwargs["os_cache"] = not kwargs["os_cache"]
    _check_arguments(kwargs, "Ocata", "bypass_url",
                     right_name="endpoint_override")
    _check_arguments(kwargs, "Ocata", "api_key", right_name="password")
    # NOTE(andreykurilin): OpenStack projects use two variables with one
    #   meaning: 'endpoint_type' and 'interface'. 'endpoint_type' is an old
    #   name which was used by most OpenStack clients. Later it was replaced by
    #  'interface' in keystone and later some other clients switched to new
    #   variable name too. In case of novaclient, there is no need to switch to
    #  'interface' variable name due too several reasons:
    #     - novaclient uses 'endpoint_type' variable name long time ago and
    #       there is no real reasons to switch to new name;
    #     - 'interface' argument is used in several shell subcommands
    #       (for example in `nova floating-ip-bulk-create`), so we will need to
    #       modify these subcommands to not conflict with global flag
    #       'interface'
    #   Actually, novaclient did not accept 'interface' before, but since we
    #   allow additional arguments(kwargs), someone can use this variable name
    #   and face issue about unexpected behavior.
    _check_arguments(kwargs, "Ocata", "interface", right_name="endpoint_type")
    _check_arguments(kwargs, "Ocata", "tenant_name", right_name="project_name")
    _check_arguments(kwargs, "Ocata", "tenant_id", right_name="project_id")
    _check_arguments(kwargs, "Ocata", "proxy_tenant_id")
    _check_arguments(kwargs, "Ocata", "proxy_token")
    _check_arguments(kwargs, "Ocata", "connection_pool")
    _check_arguments(kwargs, "Ocata", "volume_service_name")

    api_version, client_class = _get_client_class_and_version(version)
    kwargs.pop("direct_use", None)

    profile = kwargs.pop("profile", None)
    if osprofiler_profiler and profile:
        # Initialize the root of the future trace: the created trace ID will
        # be used as the very first parent to which all related traces will be
        # bound to. The given HMAC key must correspond to the one set in
        # nova-api nova.conf, otherwise the latter will fail to check the
        # request signature and will skip initialization of osprofiler on
        # the server side.
        osprofiler_profiler.init(profile)

    return client_class(api_version=api_version,
                        auth_url=auth_url,
                        direct_use=False,
                        username=username,
                        **kwargs)



# Copyright 2013 Nebula, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import base64
import subprocess


class DecryptionFailure(Exception):
    pass


def decrypt_password(private_key, password):
    """Base64 decodes password and unencrypts it with private key.

    Requires openssl binary available in the path.
    """
    unencoded = base64.b64decode(password)
    cmd = ['openssl', 'rsautl', '-decrypt', '-inkey', private_key]
    proc = subprocess.Popen(cmd, stdin=subprocess.PIPE,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
    out, err = proc.communicate(unencoded)
    proc.stdin.close()
    if proc.returncode:
        raise DecryptionFailure(err)

    if isinstance(out, bytes):
        return out.decode('utf-8')
    return out



# Copyright 2013 Rackspace Hosting
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient import api_versions
from novaclient import base


class InstanceAction(base.Resource):
    pass


class InstanceActionManager(base.ManagerWithFind):
    resource_class = InstanceAction

    def get(self, server, request_id):
        """
        Get details of an action performed on an instance.

        :param request_id: The request_id of the action to get.
        """
        return self._get("/servers/%s/os-instance-actions/%s" %
                         (base.getid(server), request_id), 'instanceAction')

    @api_versions.wraps("2.0", "2.57")
    def list(self, server):
        """
        Get a list of actions performed on a server.

        :param server: The :class:`Server` (or its ID)
        """
        return self._list('/servers/%s/os-instance-actions' %
                          base.getid(server), 'instanceActions')

    @api_versions.wraps("2.58", "2.65")
    def list(self, server, marker=None, limit=None, changes_since=None):
        """
        Get a list of actions performed on a server.

        :param server: The :class:`Server` (or its ID)
        :param marker: Begin returning actions that appear later in the action
                       list than that represented by this action request id
                       (optional).
        :param limit: Maximum number of actions to return. (optional).
                      Note the API server has a configurable default limit.
                      If no limit is specified here or limit is larger than
                      default, the default limit will be used.
        :param changes_since: List only instance actions changed later or
                              equal to a certain point of time. The provided
                              time should be an ISO 8061 formatted time.
                              e.g. 2016-03-04T06:27:59Z . (optional).
        """
        opts = {}
        if marker:
            opts['marker'] = marker
        if limit:
            opts['limit'] = limit
        if changes_since:
            opts['changes-since'] = changes_since
        return self._list('/servers/%s/os-instance-actions' %
                          base.getid(server), 'instanceActions', filters=opts)

    @api_versions.wraps("2.66")
    def list(self, server, marker=None, limit=None, changes_since=None,
             changes_before=None):
        """
        Get a list of actions performed on a server.

        :param server: The :class:`Server` (or its ID)
        :param marker: Begin returning actions that appear later in the action
                       list than that represented by this action request id
                       (optional).
        :param limit: Maximum number of actions to return. (optional).
        :param changes_since: List only instance actions changed later or
                              equal to a certain point of time. The provided
                              time should be an ISO 8061 formatted time.
                              e.g. 2016-03-04T06:27:59Z . (optional).
        :param changes_before: List only instance actions changed earlier or
                               equal to a certain point of time. The provided
                               time should be an ISO 8061 formatted time.
                               e.g. 2016-03-05T06:27:59Z . (optional).
        """
        opts = {}
        if marker:
            opts['marker'] = marker
        if limit:
            opts['limit'] = limit
        if changes_since:
            opts['changes-since'] = changes_since
        if changes_before:
            opts['changes-before'] = changes_before
        return self._list('/servers/%s/os-instance-actions' %
                          base.getid(server), 'instanceActions', filters=opts)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import requests

from novaclient import api_versions
from novaclient import base
from novaclient import exceptions
from novaclient.tests.unit import utils
from novaclient.tests.unit.v2 import fakes
from novaclient.v2 import flavors


def create_response_obj_with_header():
    resp = requests.Response()
    resp.headers['x-openstack-request-id'] = fakes.FAKE_REQUEST_ID
    return resp


def create_response_obj_with_compute_header():
    resp = requests.Response()
    resp.headers['x-compute-request-id'] = fakes.FAKE_REQUEST_ID
    return resp


class BaseTest(utils.TestCase):
    def test_resource_repr(self):
        r = base.Resource(None, dict(foo="bar", baz="spam"))
        self.assertEqual("<Resource baz=spam, foo=bar>", repr(r))

    def test_getid(self):
        self.assertEqual(4, base.getid(4))

        class TmpObject(object):
            id = 4
        self.assertEqual(4, base.getid(TmpObject))

    def test_resource_lazy_getattr(self):
        cs = fakes.FakeClient(api_versions.APIVersion("2.0"))
        f = flavors.Flavor(cs.flavors, {'id': 1})
        self.assertEqual('256 MiB Server', f.name)
        cs.assert_called('GET', '/flavors/1')

        # Missing stuff still fails after a second get
        self.assertRaises(AttributeError, getattr, f, 'blahblah')

    def test_eq(self):
        # Two resources of the same type with the same id: equal
        r1 = base.Resource(None, {'id': 1, 'name': 'hi'})
        r2 = base.Resource(None, {'id': 1, 'name': 'hello'})
        self.assertEqual(r1, r2)

        # Two resources of different types: never equal
        r1 = base.Resource(None, {'id': 1})
        r2 = flavors.Flavor(None, {'id': 1})
        self.assertNotEqual(r1, r2)

        # Two resources with no ID: equal if their info is equal
        r1 = base.Resource(None, {'name': 'joe', 'age': 12})
        r2 = base.Resource(None, {'name': 'joe', 'age': 12})
        self.assertEqual(r1, r2)

    def test_ne(self):
        # Two resources of different types: never equal
        r1 = base.Resource(None, {'id': 1, 'name': 'test'})
        r2 = object()
        self.assertNotEqual(r1, r2)

    def test_findall_invalid_attribute(self):
        cs = fakes.FakeClient(api_versions.APIVersion("2.0"))
        # Make sure findall with an invalid attribute doesn't cause errors.
        # The following should not raise an exception.
        cs.flavors.findall(vegetable='carrot')

        # However, find() should raise an error
        self.assertRaises(exceptions.NotFound,
                          cs.flavors.find,
                          vegetable='carrot')

    def test_resource_object_with_request_ids(self):
        resp_obj = create_response_obj_with_header()
        r = base.Resource(None, {"name": "1"}, resp=resp_obj)
        self.assertEqual(fakes.FAKE_REQUEST_ID_LIST, r.request_ids)

    def test_resource_object_with_compute_request_ids(self):
        resp_obj = create_response_obj_with_compute_header()
        r = base.Resource(None, {"name": "1"}, resp=resp_obj)
        self.assertEqual(fakes.FAKE_REQUEST_ID_LIST, r.request_ids)


class ListWithMetaTest(utils.TestCase):
    def test_list_with_meta(self):
        resp = create_response_obj_with_header()
        obj = base.ListWithMeta([], resp)
        self.assertEqual([], obj)
        # Check request_ids attribute is added to obj
        self.assertTrue(hasattr(obj, 'request_ids'))
        self.assertEqual(fakes.FAKE_REQUEST_ID_LIST, obj.request_ids)


class DictWithMetaTest(utils.TestCase):
    def test_dict_with_meta(self):
        resp = create_response_obj_with_header()
        obj = base.DictWithMeta({}, resp)
        self.assertEqual({}, obj)
        # Check request_ids attribute is added to obj
        self.assertTrue(hasattr(obj, 'request_ids'))
        self.assertEqual(fakes.FAKE_REQUEST_ID_LIST, obj.request_ids)


class TupleWithMetaTest(utils.TestCase):
    def test_tuple_with_meta(self):
        resp = create_response_obj_with_header()
        expected_tuple = (1, 2)
        obj = base.TupleWithMeta(expected_tuple, resp)
        self.assertEqual(expected_tuple, obj)
        # Check request_ids attribute is added to obj
        self.assertTrue(hasattr(obj, 'request_ids'))
        self.assertEqual(fakes.FAKE_REQUEST_ID_LIST, obj.request_ids)


class StrWithMetaTest(utils.TestCase):
    def test_str_with_meta(self):
        resp = create_response_obj_with_header()
        obj = base.StrWithMeta("test-str", resp)
        self.assertEqual("test-str", obj)
        # Check request_ids attribute is added to obj
        self.assertTrue(hasattr(obj, 'request_ids'))
        self.assertEqual(fakes.FAKE_REQUEST_ID_LIST, obj.request_ids)


class BytesWithMetaTest(utils.TestCase):
    def test_bytes_with_meta(self):
        resp = create_response_obj_with_header()
        obj = base.BytesWithMeta(b'test-bytes', resp)
        self.assertEqual(b'test-bytes', obj)
        # Check request_ids attribute is added to obj
        self.assertTrue(hasattr(obj, 'request_ids'))
        self.assertEqual(fakes.FAKE_REQUEST_ID_LIST, obj.request_ids)



# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from novaclient import api_versions
from novaclient.tests.unit import utils
from novaclient.tests.unit.v2 import fakes
from novaclient.v2 import volumes


class VolumesTest(utils.TestCase):
    api_version = "2.0"

    def setUp(self):
        super(VolumesTest, self).setUp()
        self.cs = fakes.FakeClient(api_versions.APIVersion(self.api_version))

    def test_create_server_volume(self):
        v = self.cs.volumes.create_server_volume(
            server_id=1234,
            volume_id='15e59938-07d5-11e1-90e3-e3dffe0c5983',
            device='/dev/vdb'
        )
        self.assert_request_id(v, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('POST', '/servers/1234/os-volume_attachments')
        self.assertIsInstance(v, volumes.Volume)

    def test_update_server_volume(self):
        vol_id = '15e59938-07d5-11e1-90e3-e3dffe0c5983'
        v = self.cs.volumes.update_server_volume(
            server_id=1234,
            src_volid='Work',
            dest_volid=vol_id
        )
        self.assert_request_id(v, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('PUT',
                              '/servers/1234/os-volume_attachments/Work')
        self.assertIsInstance(v, volumes.Volume)

    def test_get_server_volume(self):
        v = self.cs.volumes.get_server_volume(1234, 'Work')
        self.assert_request_id(v, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET',
                              '/servers/1234/os-volume_attachments/Work')
        self.assertIsInstance(v, volumes.Volume)

    def test_get_server_volume_with_exception(self):
        self.assertRaises(TypeError,
                          self.cs.volumes.get_server_volume,
                          "1234")

        self.assertRaises(TypeError,
                          self.cs.volumes.get_server_volume,
                          "1234",
                          volume_id="Work",
                          attachment_id="123")

    @mock.patch('warnings.warn')
    def test_get_server_volume_with_warn(self, mock_warn):
        self.cs.volumes.get_server_volume(1234,
                                          volume_id=None,
                                          attachment_id="Work")
        mock_warn.assert_called_once()

    def test_list_server_volumes(self):
        vl = self.cs.volumes.get_server_volumes(1234)
        self.assert_request_id(vl, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('GET',
                              '/servers/1234/os-volume_attachments')
        for v in vl:
            self.assertIsInstance(v, volumes.Volume)

    def test_delete_server_volume(self):
        ret = self.cs.volumes.delete_server_volume(1234, 'Work')
        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('DELETE',
                              '/servers/1234/os-volume_attachments/Work')


class VolumesV249Test(VolumesTest):
    api_version = "2.49"

    def test_create_server_volume_with_tag(self):
        v = self.cs.volumes.create_server_volume(
            server_id=1234,
            volume_id='15e59938-07d5-11e1-90e3-e3dffe0c5983',
            device='/dev/vdb',
            tag='test_tag'
        )
        self.assert_request_id(v, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called(
            'POST', '/servers/1234/os-volume_attachments',
            {'volumeAttachment': {
                'volumeId': '15e59938-07d5-11e1-90e3-e3dffe0c5983',
                'device': '/dev/vdb',
                'tag': 'test_tag'}})
        self.assertIsInstance(v, volumes.Volume)

    def test_delete_server_volume_with_exception(self):
        self.assertRaises(TypeError,
                          self.cs.volumes.delete_server_volume,
                          "1234")

        self.assertRaises(TypeError,
                          self.cs.volumes.delete_server_volume,
                          "1234",
                          volume_id="Work",
                          attachment_id="123")

    @mock.patch('warnings.warn')
    def test_delete_server_volume_with_warn(self, mock_warn):
        self.cs.volumes.delete_server_volume(1234,
                                             volume_id=None,
                                             attachment_id="Work")
        mock_warn.assert_called_once()


class VolumesV279Test(VolumesV249Test):
    api_version = "2.79"

    def test_create_server_volume_with_delete_on_termination(self):
        v = self.cs.volumes.create_server_volume(
            server_id=1234,
            volume_id='15e59938-07d5-11e1-90e3-e3dffe0c5983',
            device='/dev/vdb',
            tag='tag1',
            delete_on_termination=True
        )
        self.assert_request_id(v, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called(
            'POST', '/servers/1234/os-volume_attachments',
            {'volumeAttachment': {
                'volumeId': '15e59938-07d5-11e1-90e3-e3dffe0c5983',
                'device': '/dev/vdb',
                'tag': 'tag1',
                'delete_on_termination': True}})
        self.assertIsInstance(v, volumes.Volume)

    def test_create_server_volume_with_delete_on_termination_pre_v279(self):
        self.cs.api_version = api_versions.APIVersion('2.78')
        ex = self.assertRaises(
            TypeError, self.cs.volumes.create_server_volume, "1234",
            volume_id='15e59938-07d5-11e1-90e3-e3dffe0c5983',
            delete_on_termination=True)
        self.assertIn('delete_on_termination', str(ex))


class VolumesV285Test(VolumesV279Test):
    api_version = "2.85"

    def test_volume_update_server_volume(self):
        v = self.cs.volumes.update_server_volume(
            server_id=1234,
            src_volid='Work',
            dest_volid='Work',
            delete_on_termination=True
        )
        self.assert_request_id(v, fakes.FAKE_REQUEST_ID_LIST)
        self.cs.assert_called('PUT',
                              '/servers/1234/os-volume_attachments/Work')
        self.assertIsInstance(v, volumes.Volume)

    def test_volume_update_server_volume_pre_v285(self):
        self.cs.api_version = api_versions.APIVersion('2.84')
        ex = self.assertRaises(
            TypeError, self.cs.volumes.update_server_volume, "1234",
            'Work', 'Work', delete_on_termination=True)
        self.assertIn('delete_on_termination', str(ex))



# Copyright 2011 OpenStack Foundation
# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient.tests.unit.fixture_data import availability_zones as data
from novaclient.tests.unit.fixture_data import client
from novaclient.tests.unit import utils
from novaclient.tests.unit.v2 import fakes
from novaclient.v2 import availability_zones


class AvailabilityZoneTest(utils.FixturedTestCase):
    # NOTE(cyeoh): import shell here so the V3 version of
    # this class can inherit off the v3 version of shell
    from novaclient.v2 import shell  # noqa

    data_fixture_class = data.V1

    scenarios = [('original', {'client_fixture_class': client.V1}),
                 ('session', {'client_fixture_class': client.SessionV1})]

    def setUp(self):
        super(AvailabilityZoneTest, self).setUp()
        self.availability_zone_type = self._get_availability_zone_type()

    def _get_availability_zone_type(self):
        return availability_zones.AvailabilityZone

    def _assertZone(self, zone, name, status):
        self.assertEqual(zone.zoneName, name)
        self.assertEqual(zone.zoneState, status)

    def test_list_availability_zone(self):
        zones = self.cs.availability_zones.list(detailed=False)
        self.assert_request_id(zones, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/os-availability-zone')

        for zone in zones:
            self.assertIsInstance(zone, self.availability_zone_type)

        self.assertEqual(2, len(zones))

        l0 = ['zone-1', 'available']
        l1 = ['zone-2', 'not available']

        z0 = self.shell._treeizeAvailabilityZone(zones[0])
        z1 = self.shell._treeizeAvailabilityZone(zones[1])

        self.assertEqual((1, 1), (len(z0), len(z1)))

        self._assertZone(z0[0], l0[0], l0[1])
        self._assertZone(z1[0], l1[0], l1[1])

    def test_detail_availability_zone(self):
        zones = self.cs.availability_zones.list(detailed=True)
        self.assert_request_id(zones, fakes.FAKE_REQUEST_ID_LIST)
        self.assert_called('GET', '/os-availability-zone/detail')

        for zone in zones:
            self.assertIsInstance(zone, self.availability_zone_type)

        self.assertEqual(3, len(zones))

        l0 = ['zone-1', 'available']
        l1 = ['|- fake_host-1', '']
        l2 = ['| |- nova-compute', 'enabled :-) 2012-12-26 14:45:25']
        l3 = ['internal', 'available']
        l4 = ['|- fake_host-1', '']
        l5 = ['| |- nova-sched', 'enabled :-) 2012-12-26 14:45:25']
        l6 = ['|- fake_host-2', '']
        l7 = ['| |- nova-network', 'enabled XXX 2012-12-26 14:45:24']
        l8 = ['zone-2', 'not available']

        z0 = self.shell._treeizeAvailabilityZone(zones[0])
        z1 = self.shell._treeizeAvailabilityZone(zones[1])
        z2 = self.shell._treeizeAvailabilityZone(zones[2])

        self.assertEqual((3, 5, 1), (len(z0), len(z1), len(z2)))

        self._assertZone(z0[0], l0[0], l0[1])
        self._assertZone(z0[1], l1[0], l1[1])
        self._assertZone(z0[2], l2[0], l2[1])
        self._assertZone(z1[0], l3[0], l3[1])
        self._assertZone(z1[1], l4[0], l4[1])
        self._assertZone(z1[2], l5[0], l5[1])
        self._assertZone(z1[3], l6[0], l6[1])
        self._assertZone(z1[4], l7[0], l7[1])
        self._assertZone(z2[0], l8[0], l8[1])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient.tests.functional import base
from novaclient.tests.functional.v2 import fake_crypto
from novaclient.tests.functional.v2.legacy import test_keypairs


class TestKeypairsNovaClientV22(test_keypairs.TestKeypairsNovaClient):
    """Keypairs functional tests for v2.2 nova-api microversion."""

    COMPUTE_API_VERSION = "2.2"

    def test_create_keypair(self):
        keypair = super(TestKeypairsNovaClientV22, self).test_create_keypair()
        self.assertIn('ssh', keypair)

    def test_create_keypair_x509(self):
        key_name = self._create_keypair(key_type='x509')
        keypair = self._show_keypair(key_name)
        self.assertIn(key_name, keypair)
        self.assertIn('x509', keypair)

    def test_import_keypair(self):
        pub_key, fingerprint = fake_crypto.get_ssh_pub_key_and_fingerprint()
        pub_key_file = self._create_public_key_file(pub_key)
        keypair = self._test_import_keypair(fingerprint, pub_key=pub_key_file)
        self.assertIn('ssh', keypair)

    def test_import_keypair_x509(self):
        certif, fingerprint = fake_crypto.get_x509_cert_and_fingerprint()
        pub_key_file = self._create_public_key_file(certif)
        keypair = self._test_import_keypair(fingerprint, key_type='x509',
                                            pub_key=pub_key_file)
        self.assertIn('x509', keypair)


class TestKeypairsNovaClientV210(base.ProjectTestBase):
    """Keypairs functional tests for v2.10 nova-api microversion."""

    COMPUTE_API_VERSION = "2.10"

    def test_create_and_list_keypair(self):
        name = self.name_generate()
        self.nova("keypair-add %s --user %s" % (name, self.user_id))
        self.addCleanup(self.another_nova, "keypair-delete %s" % name)
        output = self.nova("keypair-list")
        self.assertRaises(ValueError, self._get_value_from_the_table,
                          output, name)
        output_1 = self.another_nova("keypair-list")
        output_2 = self.nova("keypair-list --user %s" % self.user_id)
        self.assertEqual(output_1, output_2)
        # it should be table with one key-pair
        self.assertEqual(name, self._get_column_value_from_single_row_table(
            output_1, "Name"))

        output_1 = self.another_nova("keypair-show %s " % name)
        output_2 = self.nova("keypair-show --user %s %s" % (self.user_id,
                                                            name))
        self.assertEqual(output_1, output_2)
        self.assertEqual(self.user_id,
                         self._get_value_from_the_table(output_1, "user_id"))

    def test_create_and_delete(self):
        name = self.name_generate()

        def cleanup():
            # We should check keypair existence and remove it from correct user
            # if keypair is presented
            o = self.another_nova("keypair-list")
            if name in o:
                self.another_nova("keypair-delete %s" % name)

        self.nova("keypair-add %s --user %s" % (name, self.user_id))
        self.addCleanup(cleanup)
        output = self.another_nova("keypair-list")
        self.assertEqual(name, self._get_column_value_from_single_row_table(
            output, "Name"))

        self.nova("keypair-delete %s --user %s " % (name, self.user_id))
        output = self.another_nova("keypair-list")
        self.assertRaises(
            ValueError,
            self._get_column_value_from_single_row_table, output, "Name")


class TestKeypairsNovaClientV235(base.ProjectTestBase):
    """Keypairs functional tests for v2.35 nova-api microversion."""

    COMPUTE_API_VERSION = "2.35"

    def test_create_and_list_keypair_with_marker_and_limit(self):
        names = []
        for i in range(3):
            names.append(self.name_generate())
            self.nova("keypair-add %s --user %s" % (names[i], self.user_id))
            self.addCleanup(self.another_nova, "keypair-delete %s" % names[i])

        # sort keypairs before pagination
        names = sorted(names)

        # list only one keypair after the first
        output_1 = self.another_nova("keypair-list --limit 1 --marker %s" %
                                     names[0])
        output_2 = self.nova("keypair-list --limit 1 --marker %s --user %s" %
                             (names[0], self.user_id))
        self.assertEqual(output_1, output_2)
        # it should be table with only one second key-pair
        self.assertEqual(
            names[1], self._get_column_value_from_single_row_table(output_1,
                                                                   "Name"))



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
version interface
"""

from urllib import parse

from novaclient import base
from novaclient import exceptions as exc


class Version(base.Resource):
    """
    Compute REST API information
    """
    def __repr__(self):
        return "<Version>"


class VersionManager(base.ManagerWithFind):
    resource_class = Version

    def _get_current(self):
        """Returns info about current version."""

        # TODO(sdague): we've now got to make up to 3 HTTP requests to
        # determine what version we are running, due to differences in
        # deployments and versions. We really need to cache the
        # results of this per endpoint and keep the results of it for
        # some reasonable TTL (like 24 hours) to reduce our round trip
        # traffic.
        try:
            # Assume that the value of get_endpoint() is something
            # we can get the version of. This is a 404 for Nova <
            # Mitaka if the service catalog contains project_id.
            #
            # TODO(sdague): add microversion for when this will
            # change
            url = "%s" % self.api.client.get_endpoint()
            return self._get(url, "version")
        except exc.NotFound:
            # If that's a 404, we can instead try hacking together
            # an endpoint root url by chopping off the last 2 /s.
            # This is kind of gross, but we've had this baked in
            # so long people got used to this hard coding.
            #
            # NOTE(sdague): many service providers don't really
            # implement GET / in the expected way, if we do a GET
            # /v2 that's actually a 300 redirect to
            # /v2/... because of how paste works. So adding the
            # end slash is really important.
            url = "%s/" % url.rsplit("/", 1)[0]
            return self._get(url, "version")

    def get_current(self):
        try:
            return self._get_current()
        except exc.Unauthorized:
            # NOTE(sdague): RAX's repose configuration blocks access to the
            # versioned endpoint, which is definitely non-compliant behavior.
            # However, there is no defcore test for this yet. Remove this code
            # block once we land things in defcore.
            return None

    def list(self):
        """List all versions."""

        endpoint = self.api.client.get_endpoint()
        url = parse.urlparse(endpoint)
        # NOTE(andreykurilin): endpoint URL has at least 3 formats:
        #   1. the classic (legacy) endpoint:
        #       http://{host}:{optional_port}/v{2 or 2.1}/{project-id}
        #   2. starting from microversion 2.18 project-id is not included:
        #       http://{host}:{optional_port}/v{2 or 2.1}
        #   3. under wsgi:
        #       http://{host}:{optional_port}/compute/v{2 or 2.1}
        if (url.path.endswith("v2") or "/v2/" in url.path or
                url.path.endswith("v2.1") or "/v2.1/" in url.path):
            # this way should handle all 3 possible formats
            path = url.path[:url.path.rfind("/v2")]
            version_url = '%s://%s%s' % (url.scheme, url.netloc, path)
        else:
            # NOTE(andreykurilin): probably, it is one of the next cases:
            #  * https://compute.example.com/
            #  * https://example.com/compute
            # leave as is without cropping.
            version_url = endpoint

        return self._list(version_url, "versions")



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Usage interface.
"""

import oslo_utils

from novaclient import api_versions
from novaclient import base


class Usage(base.Resource):
    """
    Usage contains information about a tenant's physical resource usage
    """
    def __repr__(self):
        return "<ComputeUsage>"

    def get(self):
        fmt = '%Y-%m-%dT%H:%M:%S.%f'
        if self.start and self.stop and self.tenant_id:
            # set_loaded() first ... so if we have to bail, we know we tried.
            self.set_loaded(True)
            start = oslo_utils.timeutils.parse_strtime(self.start, fmt=fmt)
            stop = oslo_utils.timeutils.parse_strtime(self.stop, fmt=fmt)

            new = self.manager.get(self.tenant_id, start, stop)
            if new:
                self._add_details(new._info)
                self.append_request_ids(new.request_ids)


class UsageManager(base.ManagerWithFind):
    """
    Manage :class:`Usage` resources.
    """
    resource_class = Usage
    usage_prefix = 'os-simple-tenant-usage'

    def _usage_query(self, start, end, marker=None, limit=None, detailed=None):
        query = "?start=%s&end=%s" % (start.isoformat(), end.isoformat())
        if limit:
            query = "%s&limit=%s" % (query, int(limit))
        if marker:
            query = "%s&marker=%s" % (query, marker)
        if detailed is not None:
            query = "%s&detailed=%s" % (query, int(bool(detailed)))
        return query

    @api_versions.wraps("2.0", "2.39")
    def list(self, start, end, detailed=False):
        """
        Get usage for all tenants

        :param start: :class:`datetime.datetime` Start date in UTC
        :param end: :class:`datetime.datetime` End date in UTC
        :param detailed: Whether to include information about each
                         instance whose usage is part of the report
        :rtype: list of :class:`Usage`.
        """
        query_string = self._usage_query(start, end, detailed=detailed)
        url = '/%s%s' % (self.usage_prefix, query_string)
        return self._list(url, 'tenant_usages')

    @api_versions.wraps("2.40")
    def list(self, start, end, detailed=False, marker=None, limit=None):
        """
        Get usage for all tenants

        :param start: :class:`datetime.datetime` Start date in UTC
        :param end: :class:`datetime.datetime` End date in UTC
        :param detailed: Whether to include information about each
                         instance whose usage is part of the report
        :param marker: Begin returning usage data for instances that appear
                       later in the instance list than that represented by
                       this instance UUID (optional).
        :param limit: Maximum number of instances to include in the usage
                      (optional). Note the API server has a configurable
                      default limit. If no limit is specified here or limit
                      is larger than default, the default limit will be used.
        :rtype: list of :class:`Usage`.
        """
        query_string = self._usage_query(start, end, marker, limit, detailed)
        url = '/%s%s' % (self.usage_prefix, query_string)
        return self._list(url, 'tenant_usages')

    @api_versions.wraps("2.0", "2.39")
    def get(self, tenant_id, start, end):
        """
        Get usage for a specific tenant.

        :param tenant_id: Tenant ID to fetch usage for
        :param start: :class:`datetime.datetime` Start date in UTC
        :param end: :class:`datetime.datetime` End date in UTC
        :rtype: :class:`Usage`
        """
        query_string = self._usage_query(start, end)
        url = '/%s/%s%s' % (self.usage_prefix, tenant_id, query_string)
        return self._get(url, 'tenant_usage')

    @api_versions.wraps("2.40")
    def get(self, tenant_id, start, end, marker=None, limit=None):
        """
        Get usage for a specific tenant.

        :param tenant_id: Tenant ID to fetch usage for
        :param start: :class:`datetime.datetime` Start date in UTC
        :param end: :class:`datetime.datetime` End date in UTC
        :param marker: Begin returning usage data for instances that appear
                       later in the instance list than that represented by
                       this instance UUID (optional).
        :param limit: Maximum number of instances to include in the usage
                      (optional). Note the API server has a configurable
                      default limit. If no limit is specified here or limit
                      is larger than default, the default limit will be used.
        :rtype: :class:`Usage`
        """
        query_string = self._usage_query(start, end, marker, limit)
        url = '/%s/%s%s' % (self.usage_prefix, tenant_id, query_string)
        return self._get(url, 'tenant_usage')



#    Copyright 2015 Huawei Technology corp.
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from novaclient.tests.functional import base


class TestServerGroupClient(base.ClientTestBase):
    """Server groups v2.1 functional tests."""

    COMPUTE_API_VERSION = "2.1"

    def _create_sg(self, policy):
        sg_name = self.name_generate()
        output = self.nova('server-group-create %s %s' % (sg_name, policy))
        sg_id = self._get_column_value_from_single_row_table(output, "Id")
        return sg_id

    def test_create_server_group(self):
        sg_id = self._create_sg("affinity")
        self.addCleanup(self.nova, 'server-group-delete %s' % sg_id)
        sg = self.nova('server-group-get %s' % sg_id)
        result = self._get_column_value_from_single_row_table(sg, "Id")
        self.assertEqual(sg_id, result)

    def test_list_server_group(self):
        sg_id = self._create_sg("affinity")
        self.addCleanup(self.nova, 'server-group-delete %s' % sg_id)
        sg = self.nova('server-group-list')
        result = self._get_column_value_from_single_row_table(sg, "Id")
        self.assertEqual(sg_id, result)

    def test_delete_server_group(self):
        sg_id = self._create_sg("affinity")
        sg = self.nova('server-group-get %s' % sg_id)
        result = self._get_column_value_from_single_row_table(sg, "Id")
        self.assertIsNotNone(result)
        self.nova('server-group-delete %s' % sg_id)
